@@ -30,10 +30,12 @@ static void die_unless_privileged(void);
 static void do_sanity_checks(void);
 static void session_init(struct vsf_session* p_sess);
 static void env_init(void);
+static void limits_init(void);
 
 int
 main(int argc, const char* argv[])
 {
+  tunables_load_defaults();
   struct vsf_session the_session =
   {
     /* Control connection */
@@ -41,7 +43,7 @@ main(int argc, const char* argv[])
     /* Data connection */
     -1, 0, -1, 0, 0, 0, 0,
     /* Login */
-    1, INIT_MYSTR, INIT_MYSTR,
+    1, 0, INIT_MYSTR, INIT_MYSTR,
     /* Protocol state */
     0, 1, INIT_MYSTR, 0, 0,
     /* Session state */
@@ -61,7 +63,7 @@ main(int argc, const char* argv[])
     /* Home directory */
     INIT_MYSTR,
     /* Secure connection state */
-    0, 0, 0, 0, 0, 0, -1, -1,
+    0, 0, 0, 0, 0, INIT_MYSTR, 0, -1, -1,
     /* Login fails */
     0
   };
@@ -128,7 +130,7 @@ main(int argc, const char* argv[])
     vsf_sysutil_setproctitle_init(argc, argv);
   }
   /* Initialize the SSL system here if needed - saves the overhead of each
-   *  child doing this itself.
+   * child doing this itself.
    */
   if (tunable_ssl_enable)
   {
@@ -160,6 +162,8 @@ main(int argc, const char* argv[])
   session_init(&the_session);
   /* Set up "environment", e.g. process group etc. */
   env_init();
+  /* Set up resource limits. */
+  limits_init();
   /* Set up logging - must come after global init because we need the remote
    * address to convert into text
    */
@@ -180,6 +184,7 @@ main(int argc, const char* argv[])
   if (tunable_ssl_enable)
   {
     ssl_init(&the_session);
+    ssl_add_entropy(&the_session);
   }
   if (tunable_deny_email_enable)
   {
@@ -289,6 +294,12 @@ env_init(void)
   vsf_sysutil_install_null_sighandler(kVSFSysUtilSigPIPE);
 }
 
+static void
+limits_init(void)
+{
+  vsf_sysutil_set_address_space_limit(VSFTP_AS_LIMIT);
+}
+
 static void
 session_init(struct vsf_session* p_sess)
 {
