@@ -24,8 +24,8 @@ static struct hash* s_p_ip_count_hash;
 static struct hash* s_p_pid_ip_hash;
 static unsigned int s_ipaddr_size;
 
-static void handle_sigchld(int duff);
-static void handle_sighup(int duff);
+static void handle_sigchld(void*  duff);
+static void handle_sighup(void*  duff);
 static void prepare_child(int sockfd);
 static unsigned int handle_ip_count(void* p_raw_addr);
 static void drop_ip_count(void* p_raw_addr);
@@ -53,9 +53,7 @@ vsf_standalone_main(void)
       vsf_sysutil_exit(0);
     }
     /* Son, close standard FDs to avoid SSH hang-on-exit */
-    vsf_sysutil_close_failok(0);
-    vsf_sysutil_close_failok(1);
-    vsf_sysutil_close_failok(2);
+    vsf_sysutil_reopen_standard_fds();
     vsf_sysutil_make_session_leader();
   }
   if (tunable_listen)
@@ -76,8 +74,8 @@ vsf_standalone_main(void)
   {
     vsf_sysutil_setproctitle("LISTENER");
   }
-  vsf_sysutil_install_async_sighandler(kVSFSysUtilSigCHLD, handle_sigchld);
-  vsf_sysutil_install_async_sighandler(kVSFSysUtilSigHUP, handle_sighup);
+  vsf_sysutil_install_sighandler(kVSFSysUtilSigCHLD, handle_sigchld, 0, 1);
+  vsf_sysutil_install_sighandler(kVSFSysUtilSigHUP, handle_sighup, 0, 1);
   if (tunable_listen)
   {
     struct vsf_sysutil_sockaddr* p_sockaddr = 0;
@@ -142,12 +140,8 @@ vsf_standalone_main(void)
     void* p_raw_addr;
     int new_child;
     int new_client_sock;
-    vsf_sysutil_unblock_sig(kVSFSysUtilSigCHLD);
-    vsf_sysutil_unblock_sig(kVSFSysUtilSigHUP);
     new_client_sock = vsf_sysutil_accept_timeout(
         listen_sock, p_accept_addr, 0);
-    vsf_sysutil_block_sig(kVSFSysUtilSigCHLD);
-    vsf_sysutil_block_sig(kVSFSysUtilSigHUP);
     if (vsf_sysutil_retval_is_error(new_client_sock))
     {
       continue;
@@ -177,6 +171,7 @@ vsf_standalone_main(void)
     else
     {
       /* Child context */
+      vsf_set_die_if_parent_dies();
       vsf_sysutil_close(listen_sock);
       prepare_child(new_client_sock);
       /* By returning here we "launch" the child process with the same
@@ -224,7 +219,7 @@ drop_ip_count(void* p_raw_addr)
 }
 
 static void
-handle_sigchld(int duff)
+handle_sigchld(void* duff)
 {
   unsigned int reap_one = 1;
   (void) duff;
@@ -246,10 +241,11 @@ handle_sigchld(int duff)
 }
 
 static void
-handle_sighup(int duff)
+handle_sighup(void* duff)
 {
   (void) duff;
   /* We don't crash the out the listener if an invalid config was added */
+  tunables_load_defaults();
   vsf_parseconf_load_file(0, 0);
 }
 
